@@ -1,6 +1,6 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
@@ -33,10 +33,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040a02
-pyqt_version_str = "4.10.2"
+pyqt_version = 0x040b02
+pyqt_version_str = "4.11.2"
 
-sip_min_version = 0x040e03
+sip_min_version = 0x041000
 
 qt_version = 0
 qt_edition = ""
@@ -49,7 +49,7 @@ qt_datadir = None
 qt_archdatadir = None
 qt_pluginsdir = None
 qt_xfeatures = None
-qt_shared = ""
+qt_shared = ''
 qt_framework = 0
 
 # This default value will be used by the code that bootstraps the Qt
@@ -210,10 +210,10 @@ def create_optparser():
 
     if sys.platform == 'darwin':
         g = optparse.OptionGroup(p, title="MacOS X Configuration")
-        g.add_option("--use-arch", action="store", metavar="ARCH",
+        g.add_option("--use-arch", action="append", metavar="ARCH",
                 dest="use_arch", choices=["i386", "x86_64", "ppc"],
-                help="the architecture to use when running pyuic4 "
-                        "[default: system default]")
+                help="add ARCH to the list of architectures to use when "
+                        "running pyuic4 [default: system default]")
         p.add_option_group(g)
 
     # Installation.
@@ -323,7 +323,8 @@ class ConfigurePyQt4:
             0x040800: "Qt_4_7_2",
             0x040803: "Qt_4_8_0",
             0x040804: "Qt_4_8_3",
-            0x050000: "Qt_4_8_4",
+            0x040806: "Qt_4_8_4",
+            0x050000: "Qt_4_8_6",
             0x060000: "Qt_5_0_0"
         }
 
@@ -356,18 +357,20 @@ class ConfigurePyQt4:
             check_module("QtDBus", "qdbusconnection.h",
                     "QDBusConnection::systemBus()")
 
-        check_module("QtDeclarative", "qdeclarativeview.h",
-                "new QDeclarativeView()")
+        if qt_version < 0x050000 or opts.with_deprecated:
+            check_module("QtDeclarative", "qdeclarativeview.h",
+                    "new QDeclarativeView()")
+            check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
+            check_module("QtScriptTools", "qscriptenginedebugger.h",
+                    "new QScriptEngineDebugger()")
+            check_module("QtXml", "qdom.h", "new QDomDocument()")
+
         check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
-        check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
-        check_module("QtScriptTools", "qscriptenginedebugger.h",
-                "new QScriptEngineDebugger()")
         check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
                 extra_libs=sql_libs)
         check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
         check_module("QtTest", "QtTest", "QTest::qSleep(0)")
         check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
-        check_module("QtXml", "qdom.h", "new QDomDocument()")
         check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
         check_module("phonon", "phonon/videowidget.h",
                 "new Phonon::VideoWidget()")
@@ -375,7 +378,7 @@ class ConfigurePyQt4:
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
                 extra_libs=ass_libs)
 
-        if not qt_shared:
+        if qt_shared == '':
             sipconfig.inform("QtDesigner module disabled with static Qt libraries.")
         else:
             check_module("QtDesigner", "QExtensionFactory",
@@ -469,8 +472,6 @@ class ConfigurePyQt4:
                         extra_lib_dirs=[qpy_lib_dir], extra_libs=[qpy_lib])
 
         if "QtOpenGL" in pyqt_modules:
-            generate_OpenGL_extras()
-
             qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtOpenGL", "qpyopengl")
 
             if opts.bigqt:
@@ -881,7 +882,11 @@ include(%s)
 
         if sys.platform == 'darwin':
             gui = True
-            use_arch = opts.use_arch
+
+            if opts.use_arch is None:
+                use_arch = ''
+            else:
+                use_arch = ' '.join(opts.use_arch)
         else:
             gui = False
             use_arch = ''
@@ -909,45 +914,41 @@ include(%s)
         makefile.generate()
         tool.append("pyuic")
 
-        if "QtXml" in pyqt_modules:
-            sipconfig.inform("Creating pylupdate4 Makefile...")
+        sipconfig.inform("Creating pylupdate4 Makefile...")
 
-            cxxflags_app = sipcfg.build_macros().get("CXXFLAGS_APP", "")
+        cxxflags_app = sipcfg.build_macros().get("CXXFLAGS_APP", "")
 
-            makefile = sipconfig.ProgramMakefile(
-                configuration=sipcfg,
-                build_file=os.path.join(src_dir, "pylupdate", "pylupdate.sbf"),
-                dir="pylupdate",
-                install_dir=opts.pyqtbindir,
-                console=1,
-                qt=["QtCore", "QtXml"],
-                debug=opts.debug,
-                warnings=1,
-                universal=sipcfg.universal,
-                arch=sipcfg.arch,
-                deployment_target=sipcfg.deployment_target
-            )
+        makefile = sipconfig.ProgramMakefile(
+            configuration=sipcfg,
+            build_file=os.path.join(src_dir, "pylupdate", "pylupdate.sbf"),
+            dir="pylupdate",
+            install_dir=opts.pyqtbindir,
+            console=1,
+            qt=["QtCore", "QtXml"],
+            debug=opts.debug,
+            warnings=1,
+            universal=sipcfg.universal,
+            arch=sipcfg.arch,
+            deployment_target=sipcfg.deployment_target
+        )
 
-            makefile.extra_include_dirs.append(
-                    os.path.join(src_dir, "pylupdate"))
+        makefile.extra_include_dirs.append(os.path.join(src_dir, "pylupdate"))
 
-            if cxxflags_app != "":
-                makefile.extra_cxxflags.append(cxxflags_app)
+        if cxxflags_app != "":
+            makefile.extra_cxxflags.append(cxxflags_app)
 
-            makefile.generate()
-            tool.append("pylupdate")
+        makefile.generate()
+        tool.append("pylupdate")
 
-            sipconfig.inform("Creating pyrcc4 Makefile...")
+        sipconfig.inform("Creating pyrcc4 Makefile...")
 
-            makefile = pyrccMakefile()
+        makefile = pyrccMakefile()
 
-            if cxxflags_app != "":
-                makefile.extra_cxxflags.append(cxxflags_app)
+        if cxxflags_app != "":
+            makefile.extra_cxxflags.append(cxxflags_app)
 
-            makefile.generate()
-            tool.append("pyrcc")
-        else:
-            sipconfig.inform("pylupdate4 and pyrcc4 will not be built because the Qt XML module is missing.")
+        makefile.generate()
+        tool.append("pyrcc")
 
         if opts.designer_plugin and "QtDesigner" in pyqt_modules:
             py_major = sipcfg.py_version >> 16
@@ -979,6 +980,8 @@ include(%s)
                 if dynamic_pylib:
                     if glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor)):
                         lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
+                    elif 'MULTIARCH' in ducfg and glob.glob('%s/lib/%s/libpython%d.%d*' % (ducfg['exec_prefix'], ducfg['MULTIARCH'], py_major, py_minor)):
+                        lib_dir_flag = quote('-L%s/lib/%s' % (ducfg['exec_prefix'], ducfg['MULTIARCH']))
                     elif glob.glob("%s/libpython%d.%d*" % (ducfg["LIBDIR"], py_major, py_minor)):
                         lib_dir_flag = quote("-L%s" % ducfg["LIBDIR"])
                     else:
@@ -988,7 +991,7 @@ include(%s)
 
                     link = "%s -lpython%d.%d%s" % (lib_dir_flag, py_major, py_minor, abi)
                 else:
-                    sipconfig.inform("Qt Designer plugin disabled because Python library is static")
+                    sipconfig.inform("The Qt Designer plugin was disabled because a dynamic Python library couldn't be found")
                     opts.designer_plugin = False
 
                 pysh_lib = ducfg.get("LDLIBRARY", "")
@@ -1196,73 +1199,6 @@ def remove_file(fname):
         pass
 
 
-def generate_OpenGL_extras():
-    """Generate the extras needed by the QtOpenGL module (i.e. the .sip file
-    defining the correct typedefs for the OpenGL data types.
-    """
-    sipconfig.inform("Determining the OpenGL data types...")
-
-    src = "opengl_extras.cpp"
-
-    f = open(src, "w")
-
-    f.write(
-"""#include <QFile>
-#include <QTextStream>
-#include <qgl.h>
-
-int main(int, char **)
-{
-    QFile outf("./sip/QtOpenGL/opengl_types.sip");
-
-    if (!outf.open(QIODevice::WriteOnly|QIODevice::Truncate|QIODevice::Text))
-        return 1;
-
-    QTextStream out(&outf);
-
-    if (sizeof (long) == sizeof (GLint))
-        out << "typedef long GLint;\\n";
-    else
-        out << "typedef int GLint;\\n";
-
-    if (sizeof (unsigned long) == sizeof (GLuint))
-        out << "typedef unsigned long GLuint;\\n";
-    else
-        out << "typedef unsigned GLuint;\\n";
-
-    if (sizeof (unsigned long) == sizeof (GLenum))
-        out << "typedef unsigned long GLenum;\\n";
-    else
-        out << "typedef unsigned GLenum;\\n";
-
-    if (sizeof (unsigned long) == sizeof (GLbitfield))
-        out << "typedef unsigned long GLbitfield;\\n";
-    else
-        out << "typedef unsigned GLbitfield;\\n";
-
-    out << "typedef float GLfloat;\\n";
-
-    return 0;
-}
-""")
-
-    f.close()
-
-    cmd = compile_qt_program(src, "QtOpenGL")
-
-    if cmd is None:
-        sipconfig.error("Unable to determine the OpenGL data types.")
-
-    # SIP's build system (specifically ProgramMakefile.build_command()) assumes
-    # that an executable can be created by running a single command.  In the
-    # case of MSVC this is incorrect.  As a quick hack we do the extra step
-    # here if it looks like it is needed.
-    if os.access("opengl_extras.manifest", os.F_OK):
-        run_command("mt -nologo -manifest opengl_extras.manifest -outputresource:opengl_extras.exe;1")
-
-    run_command(cmd)
-
-
 def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None, extra_libs=None):
     """Compile a simple Qt application.
 
@@ -1277,9 +1213,8 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     """
     opengl = (mname == "QtOpenGL")
 
-    qt = [mname]
-    if mname in ("QtAssistant", "QtHelp", "QtOpenGL", "QtWebKit"):
-        qt.append("QtCore")
+    qt = []
+    needed_qt_libs(mname, qt)
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
             opengl=opengl, debug=opts.debug, arch=sipcfg.arch,
@@ -1756,7 +1691,7 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
 
     if qt_version >= 0x050000 and opts.with_deprecated:
         # The name of this macro is very confusing.
-        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x040900')
+        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x04ffff')
 
     makefile.generate()
 
@@ -1888,7 +1823,7 @@ def get_build_macros(overrides):
     if "QMAKESPEC" in list(os.environ.keys()):
         fname = os.environ["QMAKESPEC"]
 
-        if not os.path.dirname(fname):
+        if not os.path.dirname(fname) or fname.startswith("unsupported"):
             qt_macx_spec = fname
             fname = os.path.join(qt_archdatadir, "mkspecs", fname)
     elif sys.platform == "darwin":
@@ -2059,12 +1994,10 @@ SOURCES = %s
 #include <QLibraryInfo>
 #include <QTextStream>
 
-// These seem to be missing from the Qt v5 beta.
-#if !defined(QT_EDITION_DESKTOP)
-#define QT_EDITION_DESKTOP      8
-#endif
-#if !defined(QT_EDITION_OPENSOURCE)
-#define QT_EDITION_OPENSOURCE   8
+// This is for Qt v5.  The value isn't accurate but triggers the desired
+// description of the edition.
+#if !defined(QT_EDITION)
+#define QT_EDITION      0x200
 #endif
 
 int main(int argc, char **argv)
@@ -2097,7 +2030,7 @@ int main(int argc, char **argv)
 #if defined(QT_SHARED) || defined(QT_DLL)
     out << "shared\\n";
 #else
-    out << "\\n";
+    out << "static\\n";
 #endif
 
     // Determine which features should be disabled.
@@ -2162,8 +2095,9 @@ int main(int argc, char **argv)
     out << "PyQt_NoOpenGLES\\n";
 #endif
 
-    if (sizeof (qreal) != sizeof (double))
-        out << "PyQt_qreal_double\\n";
+#if defined(QT_NO_FPU) || defined(QT_ARCH_ARM) || defined(QT_ARCH_WINDOWSCE) || defined(QT_ARCH_SYMBIAN) || defined(QT_ARCH_VXWORKS)
+    out << "PyQt_qreal_double\\n";
+#endif
 
     return 0;
 }
@@ -2225,7 +2159,9 @@ int main(int argc, char **argv)
     qt_xfeatures = lines[11:]
 
     if opts.assume_shared:
-        qt_shared = "shared"
+        qt_shared = 'shared'
+    elif qt_shared == 'static':
+        qt_shared = ''
 
     # 'Nokia' is the value that is used by Maemo's version of Qt.
     if qt_licensee in ('Open Source', 'Nokia'):
