@@ -1,6 +1,6 @@
 // This defines the interfaces for the pyqtBoundSignal type.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -34,7 +29,7 @@
 
 #include <Python.h>
 
-#include "qpycore_chimera.h"
+#include "qpycore_pyqtsignal.h"
 #include "qpycore_namespace.h"
 
 
@@ -50,16 +45,13 @@ typedef struct {
     PyObject_HEAD
 
     // The unbound signal.
-    PyObject *unbound_signal;
+    qpycore_pyqtSignal *unbound_signal;
 
     // A borrowed reference to the wrapped QObject that is bound to the signal.
     PyObject *bound_pyobject;
 
     // The QObject that is bound to the signal.
     QObject *bound_qobject;
-
-    // The bound signal overload.
-    Chimera::Signature *bound_overload;
 } qpycore_pyqtBoundSignal;
 
 
@@ -68,8 +60,8 @@ extern PyTypeObject qpycore_pyqtBoundSignal_Type;
 }
 
 
-PyObject *qpycore_pyqtBoundSignal_New(PyObject *unbound_signal,
-        PyObject *bound_pyobject, QObject *bound_qobject, int signal_index);
+PyObject *qpycore_pyqtBoundSignal_New(qpycore_pyqtSignal *unbound_signal,
+        PyObject *bound_pyobject, QObject *bound_qobject);
 
 
 #endif
