@@ -1,6 +1,6 @@
 // This is the signal/slot helper code for SIP.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,6 +33,7 @@
 #include <QObject>
 
 #include "qpycore_chimera.h"
+#include "qpycore_pyqtboundsignal.h"
 #include "qpycore_pyqtproxy.h"
 #include "qpycore_pyqtpyobject.h"
 #include "qpycore_sip.h"
@@ -147,7 +143,8 @@ extern "C" void *sipQtCreateUniversalSignal(void *tx, const char **sigp)
 
 
 // Factory function to create a universal slot instance.  Returns a pointer to
-// the instance or 0 if there was an error.
+// the instance or 0 if there was an error.  Note that we may also return a
+// Qt signal (from a bound signal).
 extern "C" void *sipQtCreateUniversalSlot(sipWrapper *tx, const char *sig,
         PyObject *rxObj, const char *slot, const char **member, int flags)
 {
@@ -155,11 +152,25 @@ extern "C" void *sipQtCreateUniversalSlot(sipWrapper *tx, const char *sig,
     PyObject *qrxObj;
 
     if (slot)
+    {
         qrxObj = rxObj;
+    }
+    else if (Py_TYPE(rxObj) == &qpycore_pyqtBoundSignal_Type)
+    {
+        qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)rxObj;
+
+        *member = bs->unbound_signal->signature->signature.constData();
+
+        return bs->bound_qobject;
+    }
     else if (PyMethod_Check(rxObj))
+    {
         qrxObj = PyMethod_GET_SELF(rxObj);
+    }
     else
+    {
         qrxObj = 0;
+    }
 
     QObject *qrx = 0;
 
@@ -181,7 +192,7 @@ extern "C" void *sipQtCreateUniversalSlot(sipWrapper *tx, const char *sig,
 
     res = new PyQtProxy(tx, sig, rxObj, slot, member, flags);
 
-    if (res->real_slot.signature)
+    if (res->metaObject())
     {
         // If the receiver is a QObject then move the proxy to the same thread.
         if (qrx)
@@ -250,7 +261,7 @@ extern "C" int sipQtConnect(void *tx, const char *sig, void *rx, const char *slo
     int res;
 
     Py_BEGIN_ALLOW_THREADS
-    res = QObject::connect(reinterpret_cast<QObject *>(tx), sig,
+    res = (bool)QObject::connect(reinterpret_cast<QObject *>(tx), sig,
                            reinterpret_cast<QObject *>(rx), slot,
                            (Qt::ConnectionType)type);
     Py_END_ALLOW_THREADS
@@ -318,7 +329,9 @@ extern "C" sipSlot *sipQtFindSipslot(void *tx, void **context)
 }
 
 
-// Emit the given signal from the given object.
+// Emit the given signal from the given object.  Note that we do not apply the
+// hacks when building against Qt5 that workaround the problems with signals
+// with optional arguments.  New style signals must be used instead.
 bool qpycore_qobject_emit(QObject *qtx, const char *sig, PyObject *sigargs)
 {
     // We need to explicitly check for anything that uses a proxy, so we might
@@ -358,7 +371,8 @@ bool qpycore_qobject_emit(QObject *qtx, const char *sig, PyObject *sigargs)
     if (!parsed_signature)
         return false;
 
-    bool ok = qpycore_emit(qtx, signal_index, parsed_signature, sigargs);
+    bool ok = qpycore_emit(qtx, signal_index, parsed_signature,
+            parsed_signature->py_signature.constData(), sigargs);
     delete parsed_signature;
 
     return ok;
@@ -367,20 +381,15 @@ bool qpycore_qobject_emit(QObject *qtx, const char *sig, PyObject *sigargs)
 
 // Emit a signal based on a parsed signature.
 bool qpycore_emit(QObject *qtx, int signal_index,
-        const Chimera::Signature *parsed_signature, PyObject *sigargs)
+        const Chimera::Signature *parsed_signature, const char *docstring,
+        PyObject *sigargs)
 {
     const QList<const Chimera *> &args = parsed_signature->parsed_arguments;
 
     if (args.size() != PyTuple_GET_SIZE(sigargs))
     {
-        const char *sig = parsed_signature->signature.constData();
-
-        // Bound signals include the type character.
-        if (*sig == '2')
-            ++sig;
-
         PyErr_Format(PyExc_TypeError,
-                "signal %s has %d argument(s) but %d provided", sig,
+                "%s signal has %d argument(s) but %d provided", docstring,
                 args.size(), (int)PyTuple_GET_SIZE(sigargs));
 
         return false;
@@ -401,29 +410,10 @@ bool qpycore_emit(QObject *qtx, int signal_index,
 
         if (!val)
         {
-            const char *sig;
-
-            // Use the docstring if there is one and it is auto-generated.
-            sig = parsed_signature->docstring;
-
-            if (!sig || *sig != '\1')
-            {
-                sig = parsed_signature->signature.constData();
-
-                // Bound signals include the type character.
-                if (*sig == '2')
-                    ++sig;
-            }
-            else
-            {
-                // Skip the auto-generated marker.
-                ++sig;
-            }
-
             // Mimic SIP's exception text.
             PyErr_Format(PyExc_TypeError,
-                    "%s: argument %d has unexpected type '%s'", sig,
-                    a + 1, Py_TYPE(arg_obj)->tp_name);
+                    "%s.emit(): argument %d has unexpected type '%s'",
+                    docstring, a + 1, Py_TYPE(arg_obj)->tp_name);
 
             delete[] argv;
             qDeleteAll(values.constBegin(), values.constEnd());
@@ -438,7 +428,11 @@ bool qpycore_emit(QObject *qtx, int signal_index,
     }
 
     Py_BEGIN_ALLOW_THREADS
+#if QT_VERSION >= 0x050000
+    QMetaObject::activate(qtx, signal_index, argv);
+#else
     QMetaObject::activate(qtx, signal_index, signal_index, argv);
+#endif
     Py_END_ALLOW_THREADS
 
     delete[] argv;
